# Copyright 2010, 2011 Martin Pool
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# <http://www.apache.org/licenses/LICENSE-2.0>
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Crude http client-side cache."""


import hashlib
import os
import os.path

import glib


class ImageCache(object):

    def __init__(self):
        self.cache_dir = os.path.join(
            glib.get_user_cache_dir(),
            'blurtr',
            'images')
        if not os.path.exists(self.cache_dir):
            os.makedirs(self.cache_dir, mode=0700)

    def hash_url(self, url):
        m = hashlib.md5()
        m.update(url)
        return m.hexdigest()

    def url_path(self, url):
        return os.path.join(self.cache_dir, self.hash_url(url))

    def get_present_cache_filename(self, url):
        p = self.url_path(url)
        if os.path.exists(p):
            return p

    def open_reader(self, url):
        """Return a file reading the contents of URL, or None."""
        p = self.url_path(url)
        if os.path.exists(p):
            return open(p, 'rb')

    def open_writer(self, url):
        return CacheWriter(self.url_path(url))


class CacheWriter(object):

    def __init__(self, to_path):
        self.to_path = to_path
        self.f = open(to_path + '.tmp', 'wb')

    def write(self, bytes):
        self.f.write(bytes)

    def close(self):
        self.f.close()
        os.rename(self.to_path + '.tmp', self.to_path)
