# Copyright 2010, 2011 Martin Pool
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# <http://www.apache.org/licenses/LICENSE-2.0>
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Abstraction that accesses Tumblr feeds."""


from xml.dom import (
    minidom,
    )

from blurtr.post import TumblrPost


class TumblrFeed(object):

    def __init__(self):
        self.posts = []

    def dump_posts(self):
        for p in self.posts:
            print 'post <%s> type %s' % (p.url, p.type)
            for attr in ['photo_caption']:
                v = getattr(p, attr, None)
                if v is not None:
                    print '%s: %s' % (attr, v[:50])
            if p.type == 'photo':
                for max_width, url in p.photo_urls:
                    print '%8d %s' % (max_width, url)
            print


def get_dom_text(elem):
    return ''.join(
        c.data for c in elem.childNodes
        if c.nodeType == c.TEXT_NODE)


def parse_feed_from_bytes(feed_bytes):
    dom = minidom.parseString(feed_bytes)
    ## print dom.toprettyxml(indent='  ')
    feed = TumblrFeed()
    loghead = dom.getElementsByTagName('tumblelog')[0]
    feed.name = loghead.getAttribute('name')
    feed.title = loghead.getAttribute('title')
    for post_elem in dom.getElementsByTagName('post'):
        post = TumblrPost()
        # is there a feed identifier for this particular post?
        for post_tumblelog_elem in post_elem.getElementsByTagName('tumblelog'):
            post.title = post_tumblelog_elem.getAttribute('title')
            post.name = post_tumblelog_elem.getAttribute('name')
            break
        else:
            post.title = post.name = None
        for name in ['type', 'url', 'date']:
            if post_elem.hasAttribute(name):
                v = post_elem.getAttribute(name)
            else:
                v = None
            setattr(post, name, v)
        if post.type == 'photo':
            for caption_node in post_elem.getElementsByTagName('photo-caption'):
                post.photo_caption = get_dom_text(caption_node)
                break
            post.photo_urls = []
            # TODO: Handle multi-photo posts
            for cn in post_elem.getElementsByTagName('photo-url'):
                max_width = int(cn.getAttribute('max-width'))
                url = get_dom_text(cn).encode('ascii')
                post.photo_urls.append((max_width, url))
        feed.posts.append(post)
    return feed


def open_feed_from_location(location_string):
    """Open a feed given a local filename or a URL."""
    f = open(location_string, 'rb')
    try:
        return parse_feed_from_bytes(f.read())
    finally:
        f.close()
