# Copyright 2010, 2011 Martin Pool
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# <http://www.apache.org/licenses/LICENSE-2.0>
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pprint import pformat
import sys

from twisted.internet import (
    reactor,
    )
from twisted.internet.defer import (
    Deferred,
    )
from twisted.web.client import getPage
from twisted.python.util import println
from twisted.python.failure import (
    Failure,
    )
from twisted.python import (
    log,
    )
from twisted.web.client import Agent
from twisted.web.http_headers import Headers
from twisted.internet.protocol import Protocol

from blurtr.feed import (
    parse_feed_from_bytes,
    )


class FeedImageToLoader(Protocol):

    def __init__(self, url, finished, pixbuf_loader, cache):
        self.finished = finished
        self.pixbuf_loader = pixbuf_loader
        self.cache_writer = cache.open_writer(url)

    def dataReceived(self, bytes):
        self.pixbuf_loader.write(bytes)
        self.cache_writer.write(bytes)

    def connectionLost(self, reason):
        print 'image loader connection lost: %s' % (reason.getErrorMessage(),)
        self.pixbuf_loader.close()
        self.cache_writer.close()
        self.finished.callback(self.pixbuf_loader)


class TumblrFeedProtocol(Protocol):
    def __init__(self, finished):
        self.finished = finished
        self.accum = []

    def dataReceived(self, bytes):
        self.accum.append(bytes)

    def connectionLost(self, reason):
        print 'Finished receiving body:', reason.getErrorMessage()
        feed = parse_feed_from_bytes(''.join(self.accum))
        self.finished.callback(feed)


def print_feed(feed_object):
    for p in feed_object.posts:
        print p.url


def print_dom_element(elem):
    print elem.tagName
    attrs = elem.attributes
    for a in map(attrs.item, range(attrs.length)):
        print a.name, pformat(a.value)
    print


def request_url_to_protocol(url, protocol):
    agent = Agent(reactor)

    d = agent.request(
        'GET',
        url,
        Headers({'User-Agent': ['blurtr/0']}),
        None)

    def cbResponse(response):
        print 'Response received: %r' % (response,)
        print 'Response version:', response.version
        print 'Response code:', response.code
        print 'Response phrase:', response.phrase
        print 'Response headers:'
        headers = list(response.headers.getAllRawHeaders())
        print pformat(headers)
        if response.code == 302:
            for name, value in headers:
                if name == 'Location':
                    new_url = value[0]
                    break
            else:
                print 'no redirection url?'
            print 'redirect to %s' % new_url
            return request_url_to_protocol(new_url, protocol)
        elif response.code == 200:
            finished = Deferred()
            response.deliverBody(protocol)
            finished.addCallback(print_feed)
            return finished
        else:
            return Failure(Exception('HTTP error: %d %s from <%s>' % 
                (response.code, response.phrase, url)))

    def cbError(failure):
        log.err(failure)
        print "failed to open %s: %s" % (url, failure)
        return failure

    d.addCallback(cbResponse)
    d.addErrback(cbError)
    return d
